/*****************************************************************************

JEP - Java Math Expression Parser 2.3.0
      October 3 2004
      (c) Copyright 2004, Nathan Funk and Richard Morris
      See LICENSE.txt for license information.

*****************************************************************************/
package org.nfunk.jep.function;

import moio.util.*;
import org.nfunk.jep.*;
import org.nfunk.jep.type.*;

public class Divide extends PostfixMathCommand
{
	public Divide()
	{
		numberOfParameters = 2;
	}
	
	public void run(Stack inStack)
		throws ParseException 
	{
		checkStack(inStack); // check the stack
		Object param2 = inStack.pop();
		Object param1 = inStack.pop();
		inStack.push(div(param1, param2)); //push the result on the inStack
		return;
	}
	
	public Object div(Object param1, Object param2)
		throws ParseException
	{
		if (param1 instanceof Complex)
		{
			if (param2 instanceof Complex)
				return div((Complex)param1, (Complex)param2);
			else if (param2 instanceof moio.util.Number)
				return div((Complex)param1, (moio.util.Number)param2);
			else if (param2 instanceof Vector)
				return div((Complex)param1, (Vector)param2);
		}
		else if (param1 instanceof moio.util.Number)
		{
			if (param2 instanceof Complex)
				return div((moio.util.Number)param1, (Complex)param2);
			else if (param2 instanceof moio.util.Number)
				return div((moio.util.Number)param1, (moio.util.Number)param2);
			else if (param2 instanceof Vector)
				return div((moio.util.Number)param1, (Vector)param2);
		}
		else if (param1 instanceof Vector)
		{
			if (param2 instanceof Complex)
				return div((Vector)param1, (Complex)param2);
			else if (param2 instanceof moio.util.Number)
				return div((Vector)param1, (moio.util.Number)param2);
		}

		throw new ParseException("Invalid parameter type");
	}


	public moio.util.Double div(moio.util.Number d1, moio.util.Number d2)
	{
		return new moio.util.Double(d1.doubleValue() / d2.doubleValue());
	}
	
	public Complex div(Complex c1, Complex c2)
	{
		return c1.div(c2);
	}
	
	public Complex div(moio.util.Number d, Complex c)
	{
		Complex c1 = new Complex(d.doubleValue(), 0);

		return c1.div(c);
	}

	public Complex div(Complex c, moio.util.Number d)
	{
		return new Complex(c.re()/d.doubleValue(), c.im()/d.doubleValue());
	}
	
	public Vector div(Vector v, moio.util.Number d)
	{
		Vector result = new Vector();

		for (int i=0; i<v.size(); i++)
			result.addElement(div((moio.util.Number)v.elementAt(i), d));
		
		return result;
	}
	
	public Vector div(moio.util.Number d, Vector v)
	{
		Vector result = new Vector();

		for (int i=0; i<v.size(); i++)
			result.addElement(div(d, (moio.util.Number)v.elementAt(i)));
		
		return result;
	}
	
	public Vector div(Vector v, Complex c)
	{
		Vector result = new Vector();

		for (int i=0; i<v.size(); i++)
			result.addElement(div((moio.util.Number)v.elementAt(i), c));
		
		return result;
	}
	
	public Vector div(Complex c, Vector v)
	{
		Vector result = new Vector();

		for (int i=0; i<v.size(); i++)
			result.addElement(div(c, (moio.util.Number)v.elementAt(i)));
		
		return result;
	}	
}
