/*****************************************************************************

JEP - Java Math Expression Parser 2.3.0
      October 3 2004
      (c) Copyright 2004, Nathan Funk and Richard Morris
      See LICENSE.txt for license information.

*****************************************************************************/
package org.nfunk.jep.function;

import java.lang.Math;
import moio.util.*;
import org.nfunk.jep.*;
import org.nfunk.jep.type.*;

public class Power extends PostfixMathCommand
{
	public Power()
	{
		numberOfParameters = 2;
	}
	
	public void run(Stack inStack)
		throws ParseException 
	{
		checkStack(inStack); // check the stack
		
		Object param2 = inStack.pop();
		Object param1 = inStack.pop();
		
		inStack.push(power(param1, param2));
	}
	
	public Object power(Object param1, Object param2)
		throws ParseException
	{
		if (param1 instanceof Complex) {
			if (param2 instanceof Complex)
				return power((Complex)param1, (Complex)param2);
			else if (param2 instanceof moio.util.Number) 
				return power((Complex)param1, (moio.util.Number)param2);
		}
		else if (param1 instanceof moio.util.Number) {
			if (param2 instanceof Complex)
				return power((moio.util.Number)param1, (Complex)param2);
			else if (param2 instanceof moio.util.Number) 
				return power((moio.util.Number)param1, (moio.util.Number)param2);
		}

		throw new ParseException("Invalid parameter type");
	}
	

	public Object power(moio.util.Number d1, moio.util.Number d2)
	{
		if (d1.doubleValue()<0 && d2.doubleValue() != d2.intValue())
		{
			Complex c = new Complex(d1.doubleValue(), 0.0);
			return c.power(d2.doubleValue());
		}
		else
			return new moio.util.Double(Math.pow(d1.doubleValue(),d2.doubleValue()));
	}
	
	public Object power(Complex c1, Complex c2)
	{
		Complex temp = c1.power(c2);

		if (temp.im()==0)
			return new moio.util.Double(temp.re());
		else
			return temp;
	}
	
	public Object power(Complex c, moio.util.Number d)
	{
		Complex temp = c.power(d.doubleValue());
		
		if (temp.im()==0)
			return new moio.util.Double(temp.re());
		else
			return temp;
	}

	public Object power(moio.util.Number d, Complex c)
	{
		Complex base = new Complex(d.doubleValue(), 0.0);
		Complex temp = base.power(c);
		
		if (temp.im()==0)
			return new moio.util.Double(temp.re());
		else
			return temp;
	}
	
}
