/*****************************************************************************

JEP - Java Math Expression Parser 2.3.0
      October 3 2004
      (c) Copyright 2004, Nathan Funk and Richard Morris
      See LICENSE.txt for license information.

*****************************************************************************/
package org.nfunk.jep.function;

import moio.util.*;
import org.nfunk.jep.*;
import org.nfunk.jep.type.*;

public class Subtract extends PostfixMathCommand
{
	public Subtract()
	{
		numberOfParameters = 2;
	}
	
	public void run(Stack inStack)
		throws ParseException 
	{
		checkStack(inStack); // check the stack
		
		Object param2 = inStack.pop();
		Object param1 = inStack.pop();
		
		inStack.push(sub(param1, param2));

		return;
	}
	
	public Object sub(Object param1, Object param2)
		throws ParseException
	{
		if (param1 instanceof Complex)
		{
			if (param2 instanceof Complex)
			{
				return sub((Complex)param1, (Complex)param2);
			}
			else if( param2 instanceof moio.util.Number)
			{
				return sub((Complex)param1, (moio.util.Number)param2);
			}
		}
		else if (param1 instanceof moio.util.Number)
		{
			if (param2 instanceof Complex)
			{
				return sub((moio.util.Number)param1, (Complex)param2);
			}
			else if (param2 instanceof moio.util.Number)
			{
				return sub((moio.util.Number)param1, (moio.util.Number)param2);
			}
		} 
		throw new ParseException("Invalid parameter type");
	}
	

	public moio.util.Double sub(moio.util.Number d1, moio.util.Number d2)
	{
		return new moio.util.Double(d1.doubleValue() - d2.doubleValue());
	}
	
	public Complex sub(Complex c1, Complex c2)
	{
		return new Complex(c1.re() - c2.re(), c1.im() - c2.im());
	}
	
	public Complex sub(Complex c, moio.util.Number d)
	{
		return new Complex(c.re() - d.doubleValue(), c.im());
	}

	public Complex sub(moio.util.Number d, Complex c)
	{
		return new Complex(d.doubleValue() - c.re(), -c.im());
	}
}
